package de.tuberlin.emt.generator;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import de.tuberlin.emt.generator.data.PluginData;
import de.tuberlin.emt.generator.data.TransformationData;
import de.tuberlin.emt.generator.jet.JETGateway;
import de.tuberlin.emt.model.Transformation;


/**
 * Class : Generator <br/>
 * Package : de.tuberlin.emftransformation.generator <br/>
 * Project : de.tuberlin.emftransformation.generator <br/>
 * Created On : 20.12.2005 <br/>
 * Description : This class is responsible for the 
 * generating process of the editor plugin. <br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */
public class Generator {

	/**
	 * The gateway for the generating process.
	 */
	private JETGateway gateway = null;

	/**
	 * The encapsulated data of the VL-Alphabet. This is the upper class of the data structure
	 * so that all information of the alphabet can be accessed.
	 */
	private TransformationData transformationData;
	
	/**
	 * Data that cointains all necessary information about the plugin: Libraries, Paths and 
	 * File names.
	 */
	private PluginData pluginData;
	
	/**
	 * In this list all filenames are stored which could not be generated.
	 */
	private List failedGenerations = new Vector();
	
	public static String[] runtimeLibraries = new String[]{};

	/**
	 * The constructor for Generator main class. All needed classes for the generation process 
	 * are initialized. That includes the JETGateway and the data structure that encapsulated the
	 * alphabet. 
	 * @param progressMonitor A ProgressMonitor for showing the progress of the generation.
	 * @param spec The VL-specification loaded by the wizard. 
	 * @param projectName The project name for the plugin.
	 * @param fileext 
	 */
	public Generator(Transformation transformation,
			String projectName) {
		this.gateway = new JETGateway(projectName);
		this.transformationData = new TransformationData(transformation);
		this.pluginData = new PluginData();
		this.pluginData.setProjectName(projectName);
	}

	/**
	 * @return
	 */
	public TransformationData getTransformationData(){
		return this.transformationData;
	}

	
	/**
	 * This method starts the generation process by calling several methods. 
	 * @param monitor
	 */
	public void start(IProgressMonitor monitor) {

		//monitor.subTask("Creating manifest files");
		//monitor.setTaskName("Creating manifest files");
		createManifest(monitor);
		//monitor.worked(1);
		
		//monitor.subTask("Generating rule classes");
		generateRuleClasses(monitor);

		if (!failedGenerations.isEmpty()) {
			StringBuffer buf = new StringBuffer();
			for (Iterator it = failedGenerations.iterator(); it.hasNext();) {
				buf.append(it.next());
			}
			MessageDialog.openWarning(Display.getCurrent().getActiveShell(),
					"", buf.toString());
		}
		monitor.done();

	}
		
	private void createManifest(IProgressMonitor monitor) {
		String[] manifestFiles = {"plugin.xml","build.properties"};
		gateway.setTemplateFileExtension("xmljet");
		gateway.setIsCode(false);
		gateway.setPackagename("");
		
		try {
			gateway.createJavaProject(monitor);
		} catch (CoreException e) {
			e.printStackTrace();
			MessageDialog.openError(Display.getCurrent().getActiveShell(),"",e.getMessage()+ "\n" + e.getCause().toString() +"\n");
		}
		
		for(int i = 0; i < manifestFiles.length;i++){
			gateway.setTargetFile(manifestFiles[i]);
			gateway.setTemplateUri(manifestFiles[i]);
			try {
				
				gateway.generate(monitor, pluginData);
			} catch (CoreException e) {
				e.printStackTrace();
			MessageDialog.openError(Display.getCurrent().getActiveShell(),"",e.getMessage()+ "\n" + e.getCause().toString() +"\n");
			}
		}
	}
	
	private void createSourceCode(String templateFile, String targetPackage, String targetFile, IProgressMonitor monitor) {
		gateway.setIsCode(true);
		gateway.setTargetFile(targetFile);
		gateway.setTemplateFileExtension("javajet");
		gateway.setTemplateUri(templateFile);
		gateway.setPackagename(targetPackage);
		
		try {
			gateway.generate(monitor, transformationData);
		} catch (CoreException e) {
			e.printStackTrace();
		MessageDialog.openError(Display.getCurrent().getActiveShell(),"",e.getMessage()+ "\n" + e.getCause().toString() +"\n");
		}	
	}


	private void generateRuleClasses(IProgressMonitor monitor) {
		String projectprefix = null;
		EList packages=this.transformationData.getTransformation().getPackages();
		for (Iterator iter = packages.iterator(); iter.hasNext();) {
			Object element = (Object) iter.next();
			if (element instanceof EPackage) {
				EPackage thePackage = (EPackage) element;
				projectprefix = thePackage.getName();
				break;
			}				
		}
		if(projectprefix == null)
			projectprefix = this.pluginData.getProjectName().toLowerCase();
		transformationData.setPackageName(projectprefix);
		//transformationData.setModelName(packageName);
//		createSourceCode("rules/AbstractRule", projectprefix+"/rules", "AbstractRule", monitor);
		createSourceCode("rules/Query", projectprefix+"/queries", "Query", monitor);
		createSourceCode("rules/TargetQuery", projectprefix+"/queries", "TargetQuery", monitor);
		createSourceCode("rules/SourceQuery", projectprefix+"/queries", "SourceQuery", monitor);
		createSourceCode("rules/InjectivityQuery", projectprefix+"/queries", "InjectivityQuery", monitor);
		createSourceCode("rules/VariableQuery", projectprefix+"/queries", "VariableQuery", monitor);		
		createSourceCode("rules/TypeQuery", projectprefix+"/queries", "TypeQuery", monitor);		
		createSourceCode("rules/Matchfinder", projectprefix+"/rules", "Matchfinder", monitor);
		createSourceCode("rules/Variable", projectprefix+"/queries", "Variable", monitor);
		createSourceCode("rules/Transformation", projectprefix+"/rules", "Transformation", monitor);
		//createSourceCode("rules/Parameter", projectprefix+"/rules", "Parameter", monitor);
		for (int i = 0; i < transformationData.getRuleCount(); i++) {
			transformationData.setRuleIndex(i);
			createSourceCode("rules/Rule", projectprefix+"/rules", transformationData.getRuleByIndex(i).getClassName(), monitor);
			createSourceCode("wrapper/Wrapper", projectprefix+"/wrapper", transformationData.getRuleByIndex(i).getName()+"Wrapper", monitor);
		}
	}
}

/**
 * $Log: Generator.java,v $
 * Revision 1.1.2.1.2.5  2007/01/27 14:37:35  crema
 * removed toLowerCase method call on packagename
 *
 * Revision 1.1.2.1.2.4  2006/11/15 17:26:36  reneschm
 * changed the way of finding the correct rootpackage name
 *
 * Revision 1.1.2.1.2.3  2006/11/13 13:10:47  reneschm
 * conversion vrom projectName to packagePrefix now uses lowercases only
 *
 * Revision 1.1.2.1.2.2  2006/10/29 17:54:35  reneschm
 * added IOObjects merge with head
 *
 * Revision 1.3  2006/10/04 14:05:06  bunjip
 * Create a project specific super package
 *
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.8  2006/06/29 09:56:15  bunjip
 * Wiederherstellung alte Version
 *
 * Revision 1.6  2006/06/15 12:25:13  enrico
 * *** empty log message ***
 *
 * Revision 1.5  2006/06/04 11:16:25  bunjip
 * Umstellung Data-Klassen
 *
 * Revision 1.4  2006/05/31 13:24:23  bunjip
 * Anpassungen an neues Modell
 *
 * Revision 1.3  2006/05/24 12:50:59  bunjip
 * Code-Review durchgefuehrt - unnoetiger Code entfernt
 *
 * Revision 1.2  2006/05/24 12:45:06  bunjip
 * letzte Veraenderungen uebernommen
 */