package de.tuberlin.emt.generator.data;

import java.util.Vector;

import org.eclipse.emf.ecore.EAttribute;

import de.tuberlin.emt.common.util.TypeConverter;
import de.tuberlin.emt.model.Property;


/**
 * Class : AttributeData <br/>
 * Package : de.tuberlin.emftransformation.generator.util <br/>
 * Project : de.tuberlin.emftransformation.generator <br/>
 * Created On : 28.12.2005 <br/>
 * Description : Wrapper class to extract information from a
 * transformation model<br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */

public class AttributeData{
	private EAttribute attribute;
	private SymbolData owner;
	private Vector<VariableData> vars = new Vector<VariableData>();
	private String expression;
	private int nacIndex = -1;
	private boolean lhs = false;
	private boolean rhs = false;
	private boolean isChangeable;
	private boolean isMany;
	
	
	/**
	 * Constructor.
	 * @param attribute Attribute that is represented by this AttributeData
	 * @param owner SymbolData that contains this attribute
	 * @param index index of the NAC, or -1 if attribute doesn't appear in a NAC
	 */
	public AttributeData(EAttribute attribute, SymbolData owner, int index){
		this.attribute = attribute;
		this.isChangeable = attribute.isChangeable();
		this.isMany = attribute.isMany();
		
		this.owner = owner;
		this.nacIndex = index;
		if(nacIndex == -1){
			rhs = true;
		}
		if(nacIndex == -2){
			lhs = true;
		}
		
	}
		
	public void addVar(VariableData data){
		this.vars.add(data);
	}
	
	public void setExp(Property prop){
		this.expression = prop.getExpression();
	}
	
	/*public void setExp(String exp){
		this.expression = exp;
	}*/
	    
	/**
	 * Returns SymbolData that contains this attribute.
	 * @return SymbolData that contains this attribute
	 */
	public SymbolData getOwner(){
		return this.owner;
	}
	
	/**
	 * Returns name of this attribute.
	 * @return name of this attribute.
	 */
	public String getName(){
		return attribute.getName();
	}
	
	/**
	 * Returns name of this attribute, first caracter in upper case.
	 * @return name of this attribute, first caracter in upper case
	 */
	public String getUpName(){
		String result = attribute.getName();
		result = result.substring(0,1).toUpperCase().concat(result.substring(1));
		return result;
	}
	
	/**
	 * Returns name of the attribute that stores this attributes value in
	 * the rule class.
	 * @return name of the attribute that stores this attributes value
	 */
	public String getVariableName() {
		String result = owner.getVariableName();
		result = result.substring(0,1).toUpperCase().concat(result.substring(1));
		result = "old_" + result + "_" + getUpName();
		return result;
	}

	/**
	 * Returns type of the attribute.
	 * @return type of the attribute
	 */
	public String getType(){
		return attribute.getEAttributeType().getName();
	}
	
	public String getJavaType(){
		return TypeConverter.toJavaClass(this.attribute.getEAttributeType().getName());
	}
	
	/**
	 * Returns expression of the attribute.
	 * @return expression of the attribute
	 */
	public String getExpression(){
		return this.expression;
	}
	
	/**
	 * Returns all VariableData that are connected with this attribute.
	 * @return all VariableData that are connected with this attribute
	 */
	public Vector<VariableData> getVars(){
		return this.vars;
	}
	
	/**
	 * Returns if attribute appears in LHS.
	 * @return if attribute appears in LHS
	 */
	public boolean isLHS(){
		return lhs;
	}
	
	/**
	 * Returns if attribute appears in RHS.
	 * @return if attribute appears in RHS
	 */
	public boolean isRHS(){
		return rhs;
	}
	
	/**
	 * Returns if attribute appears in a NAC.
	 * @return if attribute appears in a NAC
	 */
	public boolean isNAC(){
		return (nacIndex>=0);
	}
	
	/**
	 * Returns if attribute appears in a NAC with the given index.
	 * @param index index of NAC
	 * @return if attribute appears in this NAC
	 */
	public boolean isNAC(int index){
		return (index==nacIndex);
	}

	public boolean isChangeable() {
		return isChangeable;
	}

	public void setChangeable(boolean isChangeable) {
		this.isChangeable = isChangeable;
	}

	public boolean isMany() {
		return isMany;
	}

	public void setMany(boolean isMany) {
		this.isMany = isMany;
	}
}

/**
 * $Log: AttributeData.java,v $
 * Revision 1.1.4.2  2006/12/15 14:21:23  reneschm
 * merged with head
 *
 * Revision 1.2  2006/10/10 10:05:49  enrico
 * toJava->toJavaClass
 *
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.6  2006/07/10 11:38:50  enrico
 * Referenz und Attributabfragen in den Generierungsprozess verschoben
 *
 * Revision 1.5  2006/06/06 17:59:36  bunjip
 * Umwandlung EMF->JavaTyp
 *
 * Revision 1.4  2006/06/06 11:51:29  bunjip
 * *** empty log message ***
 *
 * Revision 1.3  2006/06/05 11:37:14  bunjip
 * *** empty log message ***
 *
 * Revision 1.2  2006/06/05 11:20:34  bunjip
 * getExpression veraendert - Unterscheidung ob Variablenausdruck oder Konstante
 *
 * Revision 1.1  2006/06/04 11:18:24  bunjip
 * Data-Klassen (zeitweise) wieder in Generator-Projekt integriert
 *
 * Revision 1.8  2006/05/31 21:05:28  bunjip
 * Anpassungen an neues Modell
 *
 * Revision 1.7  2006/05/30 21:06:32  bunjip
 * Anpassungen an neues Modell (nur offensichtliche)
 *
 * Revision 1.6  2006/05/24 12:18:54  bunjip
 * Initialisierung fuer alle Attribute
 *
 * Revision 1.5  2006/05/23 23:35:12  bunjip
 * mehrere EObjects pro SymbolData
 *
 * Revision 1.4  2006/05/23 13:00:11  bunjip
 * Warnings beseitigt und kleinere Verbesserungen
 *
 * Revision 1.3  2006/05/23 12:35:28  bunjip
 * vorlaeufige Version (keine Syntaxfehler)
 *
 * Revision 1.2  2006/05/22 14:29:54  bunjip
 * offensichtliche und leicht zu behebende Fehler beseitigt
 *
 * Revision 1.1  2006/05/15 09:09:41  enrico
 * *** empty log message ***
 *
 * Revision 1.3  2006/04/13 20:56:05  bunjip
 * javadoc hinzugefuegt
 *
 */