package de.tuberlin.emt.generator.data;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;

/**
 * Class : LinkData <br/>
 * Package : de.tuberlin.emfrule.generator.util <br/>
 * Project : de.tuberlin.emfrule.generator <br/>
 * Created On : 28.12.2005 <br/>
 * Description : Wrapper class to extract information from a
 * transformation model<br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */

public class LinkData {
	private EReference ref;
	private SymbolData source;
	private SymbolData target = null;
	private EObject tempTarget;
	private RuleData rule;
	private int nacIndex = -1;
	private boolean lhs = false;
	private boolean rhs = false;
	
	private boolean isMany;
	private boolean isContainment;
	private boolean isContainer;
	private boolean isChangeable;
	private boolean isDerived;
	
	/**
	 * Constructor.
	 * @param link Link that is represented by this LinkData
	 * @param source SymbolData that represents the source of this link
	 * @param rule RuleData, that contains this link
	 * @param index index of the NAC that contains this link , or -1 otherwise 
	 */
	public LinkData(EReference ref, SymbolData source, RuleData rule, EObject target, int index){
		this.ref = ref;
		this.isMany = ref.isMany();
		this.isContainment = ref.isContainment();
		this.isContainer = ref.isContainer();
		this.isChangeable = ref.isChangeable();
		this.isDerived = ref.isDerived();
		
		this.source = source;
		this.rule = rule;
		this.tempTarget = target;
		this.nacIndex = index;
		if(nacIndex == -1){
			rhs = true;
		}
		if(nacIndex == -2){
			lhs = true;
		}
	}
	
	/**
	 * Returns if the link is a containment.
	 * @return if the link is a containment
	 */
	public boolean isContainment(){
		return ref.isContainment();
	}
	
	/**
	 * Returns type of the link.
	 * @return typeof the link
	 */
	public String getType(){
		return ref.getName();
	}
	
	/**
	 * Returns type of the link, with first character in upper case.
	 * @return type of the link, first character in upper case
	 */
	public String getUpType(){
		String result = ref.getName();
		result = result.substring(0,1).toUpperCase().concat(result.substring(1));
		return result;
	}
	
	/**
	 * Returns source ouf the link.
	 * @return source ouf the link
	 */
	public SymbolData getSource(){
		return this.source;
	}
	
	/**
	 * Returns target ouf the link.
	 * @return target ouf the link
	 */
	public SymbolData getTarget(){
		if(this.target == null){
			this.target = this.rule.findTarget(tempTarget);
			if(this.target!=null){
				this.target.addIncoming(this);
			}
		}
		return this.target;
	}
	
	/**
	 * Returns containing rule of the link.
	 * @return containing rule of the link
	 */
	public RuleData getRule(){
		return this.rule;
	}
	
	/**
	 * Returns if the link is created by the rule.
	 * @return if link is created by the rule
	 */
	public boolean isCreated(){
		if(!isRHS() || target == null){
			return false;
		}
		EObject sourceOrigin = source.getLHSObject();
		EObject targetOrigin = target.getLHSObject();
		if(sourceOrigin!=null && targetOrigin!=null && sourceOrigin.eGet(ref)!=null){
			if(ref.isMany()){
				if(((EList)sourceOrigin.eGet(ref)).contains(targetOrigin)) return false;
			}
			else if (sourceOrigin.eGet(ref).equals(targetOrigin)) return false;
		}
		return true;
	}
	
	/**
	 * Returns if the link is deleted by the rule.
	 * @return if link is deleted by the rule
	 */
	public boolean isDeleted(){
		if(!isLHS() || target == null){
			return false;
		}
		EObject sourceImage = source.getRHSObject();
		EObject targetImage = target.getRHSObject();
		if(targetImage!=null && sourceImage!=null && sourceImage.eGet(ref)!=null){
			if(ref.isMany()){
				if(((EList)sourceImage.eGet(ref)).contains(targetImage)) return false;
			}
			else if (sourceImage.eGet(ref).equals(targetImage)) return false;
		}
		return true;
	}
	
	/**
	 * Returns if link appears in LHS.
	 * @return if link appears in LHS
	 */
	public boolean isLHS(){
		return lhs;
	}
	
	/**
	 * Returns if link appears in RHS.
	 * @return if link appears in RHS
	 */
	public boolean isRHS(){
		return rhs;
	}
	
	/**
	 * Returns if link appears in a NAC.
	 * @return if link appears in a NAC
	 */
	public boolean isNAC(){
		return (nacIndex>=0);
	}
	
	/**
	 * Returns if link appears in a NAC with the given index.
	 * @param index index of NAC
	 * @return if link appears in this NAC
	 */
	public boolean isNAC(int index){
		return (index==nacIndex);
	}
	
	/**
	 * Returns if link has many-multiplicity.
	 * @return if link has many-multiplicity
	 */
	public boolean isMany(){
		return ref.isMany();
	}

	public void setContainment(boolean isContainment) {
		this.isContainment = isContainment;
	}

	public void setMany(boolean isMany) {
		this.isMany = isMany;
	}

	public boolean isContainer() {
		return isContainer;
	}

	public void setContainer(boolean isContainer) {
		this.isContainer = isContainer;
	}

	public boolean isChangeable() {
		return isChangeable;
	}

	public void setChangeable(boolean isChangeable) {
		this.isChangeable = isChangeable;
	}
	
	public boolean isDerived() {
		return isDerived;
	}

	public void setDerived(boolean isDerived) {
		this.isDerived = isDerived;
	}
}

/**
 * $Log: LinkData.java,v $
 * Revision 1.1.4.2  2006/12/15 14:21:23  reneschm
 * merged with head
 *
 * Revision 1.2  2006/10/25 13:01:49  bunjip
 * isDerived (incl. getter/setter) added
 *
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.4  2006/07/10 11:38:50  enrico
 * Referenz und Attributabfragen in den Generierungsprozess verschoben
 *
 * Revision 1.3  2006/06/26 13:14:49  bunjip
 * Fix fuer die unspezifierten Referenzen innerhalb von Ecore-Regeln
 *
 * Revision 1.2  2006/06/06 11:51:29  bunjip
 * *** empty log message ***
 *
 * Revision 1.1  2006/06/04 11:18:23  bunjip
 * Data-Klassen (zeitweise) wieder in Generator-Projekt integriert
 *
 * Revision 1.7  2006/05/23 23:35:12  bunjip
 * mehrere EObjects pro SymbolData
 *
 * Revision 1.6  2006/05/23 13:48:52  bunjip
 * *** empty log message ***
 *
 * Revision 1.5  2006/05/23 13:00:11  bunjip
 * Warnings beseitigt und kleinere Verbesserungen
 *
 * Revision 1.4  2006/05/23 12:35:28  bunjip
 * vorlaeufige Version (keine Syntaxfehler)
 *
 * Revision 1.3  2006/05/22 21:11:15  bunjip
 * *** empty log message ***
 *
 * Revision 1.2  2006/05/22 14:29:54  bunjip
 * offensichtliche und leicht zu behebende Fehler beseitigt
 *
 * Revision 1.1  2006/05/15 09:09:41  enrico
 * *** empty log message ***
 *
 * Revision 1.3  2006/04/13 20:45:00  bunjip
 * javadoc hinzugefuegt
 *
 */