package de.tuberlin.emt.generator.data;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;

import de.tuberlin.emt.model.InputObject;
import de.tuberlin.emt.model.LHS;
import de.tuberlin.emt.model.Mapping;
import de.tuberlin.emt.model.NAC;
import de.tuberlin.emt.model.ObjectStructure;
import de.tuberlin.emt.model.Property;
import de.tuberlin.emt.model.RHS;
import de.tuberlin.emt.model.Rule;
import de.tuberlin.emt.model.Variable;


/**
 * Class : RuleData <br/>
 * Package : de.tuberlin.emfrule.generator.util <br/>
 * Project : de.tuberlin.emfrule.generator <br/>
 * Created On : 29.12.2005 <br/>
 * Description : Wrapper class to extract information from a
 * transformation rule<br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */

public class RuleData{
	private Rule rule;
	private Vector<SymbolData> symbols = new Vector<SymbolData>();
	private Vector<InputData> inputObjects = new Vector<InputData>();
	private Vector<AttributeData> attributes = new Vector<AttributeData>();
	private Vector<LinkData> links = new Vector<LinkData>();
	private Vector<VariableData> vars = new Vector<VariableData>();
	private HashMap<EObject,SymbolData> symbolMap = new HashMap<EObject,SymbolData>();
	
	public RuleData(Rule rule){
		this.rule = rule;
		this.initSymbols();
		this.initInputObjects();
		this.initProperties();
		this.initVariables();
	}
	
	/**
	 * Create all SymbolData and set target-reference for LinkData.
	 */
	protected void initSymbols(){
		initLHSSymbols();
		initNonLHSSymbols(rule.getRHS());
		for(int i=0; i<rule.getNAC().size();i++){
			NAC nac = (NAC)rule.getNAC().get(i);
			initNonLHSSymbols(nac);
		}
		for(int i=links.size()-1; i>=0; i--) {
			SymbolData target = links.get(i).getTarget();
			if(target==null){
				links.get(i).getSource().removeOutgoing(links.get(i));
				links.remove(i);
			}
		
		}
	}
	
	private SymbolData getSymbolDataByEObject(EObject eObject) {
		for (Iterator it = symbols.iterator(); it.hasNext();) {
			SymbolData current = (SymbolData) it.next();
			if (current.getSymbol() == eObject)
				return current;
		}
		return null;
	}
	
	protected void initInputObjects() {
		LHS lhs = rule.getLHS();
		List inputObj =  lhs.getInputObjects();
		if (inputObj != null)
		for (Iterator it = inputObj.iterator(); it.hasNext();) {
			InputObject current = (InputObject) it.next();
			InputData data = new InputData(current, getSymbolDataByEObject(current.getObject()));
			inputObjects.add(data.getIndex(), data);
		}
	}
	
	/**
	 * Create a SymbolData for every Symbol of the LHS. Mapped Symbols are added to the SymbolData that is created for the LHS.
	 */
	protected void initLHSSymbols(){
		for(int i=0; i<rule.getLHS().getAllObjects().size();i++){
			EObject current = (EObject)rule.getLHS().getAllObjects().get(i);
			SymbolData currentData = new SymbolData(current, this, -2);
			int counter = 0;
			for (Iterator it = rule.getLHS().getAllObjects().iterator(); it.hasNext();) {
				EObject tempSymbol = (EObject) it.next();
				if (tempSymbol.eClass().equals(current.eClass()) && (rule.getLHS().getAllObjects().indexOf(tempSymbol) < i)){
					counter++;
				}
			}
			currentData.createVariableName(counter, false);
			symbols.add(currentData);
			symbolMap.put(current,currentData);
		}
	}
	
	/**
	 * Create a SymbolData for every Symbol in the RHS or NAC, that is not mapped.
	 * @param struct ObjectStructure that is checked for Symbols, only RHS or a NAC
	 */
	protected void initNonLHSSymbols(ObjectStructure struct){
		int index = rule.getNAC().indexOf(struct); 
		for(int i=0; i<struct.getAllObjects().size();i++){
			EObject current = (EObject)struct.getAllObjects().get(i);
			SymbolData currentData = null;
			for(int j =0; j<rule.getMappings().size();j++){
				Mapping mapp = (Mapping)rule.getMappings().get(j);
				if(mapp.getImage().equals(current)){
					currentData = findTarget(mapp.getOrigin());
					currentData.addImage(current,index);
				}
			}
			if(currentData==null){
				currentData = new SymbolData(current, this,index);
				if(struct instanceof RHS){
					int counter = 0;
					for (Iterator it = struct.getAllObjects().iterator(); it.hasNext();) {
						EObject tempSymbol = (EObject) it.next();
						if (tempSymbol.eClass().equals(current.eClass()) && (struct.getAllObjects().indexOf(tempSymbol)< i)){
							counter++;
						}						
					}
					currentData.createVariableName(counter, true);
				}
				symbols.add(currentData);
			}
			symbolMap.put(current,currentData);
		}
	}
		

	
	protected void initProperties(){
		EList properties = rule.getProperties();
		for(int i=0; i<properties.size();i++){
			Property current = (Property) properties.get(i);
			EAttribute att = (EAttribute)current.getFeature();
			EObject obj = current.getObject();
			SymbolData sData = findTarget(obj);
			AttributeData aData = null;
			if(isLHSObject(obj)){
				aData = new AttributeData(att,sData,-2);
			}
			else if(isRHSObject(obj)){
				aData = new AttributeData(att,sData,-1);
			}
			else{
				for(int j=0; j<getNACCount();j++){
					if(isNACObject(obj,j)){
						aData = new AttributeData(att,sData,j);
					}
				}
			}
			
			aData.setExp(current);
			sData.addAttribute(aData);
			this.attributes.add(aData);		
		}
	}
	
	protected void initVariables(){
		EList variables = rule.getVariables();
		for(int i=0; i<variables.size();i++){
			Variable current = (Variable) variables.get(i);
			VariableData vData = new VariableData(current,this);
			this.vars.add(vData);
			for(int j=0; j<attributes.size();j++){
				if(current.getName().equals(attributes.get(j).getExpression())){
					attributes.get(j).addVar(vData);
					vData.addAttribute(attributes.get(j));
				}
			}
		}
	}
	
	protected boolean isLHSObject(EObject obj){
		return rule.getLHS().getAllObjects().contains(obj);
	}
	
	protected boolean isNACObject(EObject obj){
		EList nacs = rule.getNAC();
		for(int i=0; i<nacs.size();i++){
			NAC nac = (NAC) nacs.get(i);
			if(nac.getAllObjects().contains(obj)){
				return true;
			}
		}
		return false;
	}
	
	protected boolean isNACObject(EObject obj, int index){
		EList nacs = rule.getNAC();
		for(int i=0; i<nacs.size();i++){
			NAC nac = (NAC) nacs.get(i);
			if(nac.getAllObjects().contains(obj)){
				return true;
			}
		}
		return false;
	}
	
	protected boolean isRHSObject(EObject obj){
		return rule.getRHS().getAllObjects().contains(obj);
	}
	
	/**
	 * Add a LinkData to the attribute links.
	 * @param link added LinkData
	 */
	protected void addLink(LinkData link){
		this.links.add(link);
	}
	
	protected void addAttribute(AttributeData att){
		this.attributes.add(att);
	}
		
	/**
	 * Returns name of this rule.
	 * @return name of this rule
	 */
	public String getName(){
		String rulename = new String();
		StringTokenizer tokenizer = new StringTokenizer(rule.getName());
		for (;tokenizer.hasMoreTokens();)
		rulename = rulename + tokenizer.nextToken();
		return rulename;
	}
	
	/**
	 * Returns name of the class that is created for this rule.
	 * @return name of the class that is created for this rule
	 */
	public String getClassName(){
		String result = getName();
		result = result.substring(0,1).toUpperCase().concat(result.substring(1));
		result = result.concat(new String("Rule"));
		return result;
	}
	
	/**
	 * Returns if dangling condition shall be applied with this rule. 
	 * @return if dangling condition must be checked
	 */
	public boolean isDangling(){
		//return true;
		return rule.getTransformation().isDangling();
	}
	
	/**
	 * Layer of the rule.
	 * @return layer of the rule
	 */
	public int getLayer(){
		return rule.getLayer();
	}
	
	/**
	 * Returns a VariableData for every variable of the rule. 
	 * @return a VariableData for every rule variable 
	 */
	public Vector<VariableData> getVars(){
		return this.vars;
	}
	
	/**
	 * Returns all VariableDatas that are not input parameters. 
	 * @return all VariableDatas that are not input parameters
	 */
	public Vector<VariableData> getVariables() {
		Vector<VariableData> result = new Vector<VariableData>();
		for (int i=0; i<vars.size();i++){
			if(!vars.get(i).isInput()){
				result.add(vars.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all input parameters.
	 * @return all input parameters
	 */
	public Vector<VariableData> getParams(){
		Vector<VariableData> result = new Vector<VariableData>();
		for (int i=0; i<vars.size();i++){
			if(vars.get(i).isInput()){
				result.add(vars.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns VariableData with the given name. 
	 * @param name name of the variable
	 * @return VariableData with given name
	 */
	public VariableData getVar(String name){
		for (int i=0; i<vars.size();i++){
			if(vars.get(i).getName().equals(name)){
				return vars.get(i);
			}
		}
		return null;
	}
	
	/**
	 * Returns number of NACs.
	 * @return number of NACs
	 */
	public int getNACCount() {
		return rule.getNAC().size();		
	}
	
	/**
	 * Returns all SymbolData that represent created objects.
	 * @return all SymbolData that represent created objects
	 */
	public Vector<SymbolData> getCreatedSymbols(){
		Vector<SymbolData> result = new Vector<SymbolData>();
		for (int i=0; i<symbols.size();i++){
			if(symbols.get(i).isCreated()){
				result.add(symbols.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all SymbolData that represent objects deleted by the rule.
	 * @return all SymbolData that represent objects deleted by the rule
	 */
	public Vector<SymbolData> getDeletedSymbols(){
		Vector<SymbolData> result = new Vector<SymbolData>();
		for (int i=0; i<symbols.size();i++){
			if(symbols.get(i).isDeleted()){
				result.add(symbols.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all LinkData that represent references set by the rule.
	 * @return all LinkData that represent references set by the rule.
	 */
	public Vector<LinkData> getCreatedLinks(){
		Vector<LinkData> result = new Vector<LinkData>();
		for (int i=0; i<links.size();i++){
			if(links.get(i).isCreated()){
				result.add(links.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all LinkData that represent references unset by the rule.
	 * @return all LinkData that represent references unset by the rule.
	 */
	public Vector<LinkData> getDeletedLinks(){
		Vector<LinkData> result = new Vector<LinkData>();
		for (int i=0; i<links.size();i++){
			if(links.get(i).isDeleted()){
				result.add(links.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns SymbolData that contains a Symbol. 
	 * @param symbol Symbol that is contained
	 * @return SymbolData that contains given Symbol
	 */
	protected SymbolData findTarget(EObject symbol){
		return symbolMap.get(symbol);
	}
	
	/**
	 * Returns all SymbolData that represent LHS-Symbols. 
	 * @return all SymbolData that represent LHS-Symbols
	 */
	public Vector<SymbolData> getLHSSymbols(){
		Vector<SymbolData> result = new Vector<SymbolData>();
		for (int i=0; i<symbols.size();i++){
			if(symbols.get(i).isLHS()){
				result.add(symbols.get(i));
			}
		}
		return result;
	}
	
	/**
	 * All SymbolData that appear in a specified NAC
	 * @param nac index of the NAC
	 * @return all SymbolData of the NAC 
	 */
	public Vector<SymbolData> getNACSymbols(int nac) {
		Vector<SymbolData> result = new Vector<SymbolData>();
		for (int i=0; i<symbols.size();i++){
			if(symbols.get(i).isNAC(nac)){
				result.add(symbols.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Index of a SymbolData inside a NAC.
	 * @param nac index of the NAC
	 * @param data checked SymbolData  
	 * @return index inside a NAC
	 */
	public int getNACIndex(int nac, SymbolData data) {
		if(!data.isNAC(nac)||data.isLHS()){
			return getSortedLHSSymbols().indexOf(data);
		}
		else{
			int counter = 0;
			Vector<SymbolData> nacSymbols = getNACSymbols(nac);
			for(int i=0;i<nacSymbols.size();i++){
				SymbolData currentData = nacSymbols.get(i);
				if(data.equals(currentData)) return counter+getLHSSymbols().size();
				if(!currentData.isLHS()) counter++;
			}
			return -1;
		}
	}
	
	/**
	 * Returns all LinkData of the LHS.
	 * @return all LinkData of the LHS
	 */
	public Vector<LinkData> getLHSLinks(){
		Vector<LinkData> result = new Vector<LinkData>();
		for (int i=0; i<links.size();i++){
			if(links.get(i).isLHS()){
				result.add(links.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all LinkData of the RHS.
	 * @return all LinkData of the RHS
	 */
	public Vector<LinkData> getRHSLinks(){
		Vector<LinkData> result = new Vector<LinkData>();
		for (int i=0; i<links.size();i++){
			if(links.get(i).isRHS()){
				result.add(links.get(i));
			}
		}
		return result;
	}
	
	
	
	/**
	 * Returns all LinkData of a NAC.
	 * @param nac index of the NAC
	 * @return all LinkData of this NAC
	 */
	public Vector<LinkData> getNACLinks(int nac) {
		Vector<LinkData> result = new Vector<LinkData>();
		for (int i=0; i<links.size();i++){
			if(links.get(i).isNAC(nac)){
				result.add(links.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all AttributeData of the LHS.
	 * @return all AttributeData of the LHS
	 */
	public Vector<AttributeData> getLHSAttributes() {
		Vector<AttributeData> result = new Vector<AttributeData>();
		for (int i=0; i<attributes.size();i++){
			if(attributes.get(i).isLHS()){
				result.add(attributes.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all AttributeData of the RHS.
	 * @return all AttributeData of the RHS
	 */
	public Vector<AttributeData> getRHSAttributes() {
		Vector<AttributeData> result = new Vector<AttributeData>();
		for (int i=0; i<attributes.size();i++){
			if(attributes.get(i).isRHS()){
				result.add(attributes.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all AttributeData of a NAC.
	 * @param nac index of the NAC
	 * @return all AttributeData of this NAC
	 */
	public Vector<AttributeData> getNACAttributes(int nac) {
		Vector<AttributeData> result = new Vector<AttributeData>();
		for (int i=0; i<attributes.size();i++){
			if(attributes.get(i).isNAC(nac)){
				result.add(attributes.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Returns all SymbolData of this rule.
	 * @return all SymbolData of this rule
	 */
	public Vector<SymbolData> getSymbols(){
		return symbols;
	}
	
	/**
	 * Returns all SymbolData of this rule, sorted by weight.
	 * @return all SymbolData sorted by weight
	 */
	public Vector<SymbolData> getSortedLHSSymbols() {
		return sortSymbols(getLHSSymbols());
	}
	
	/**
	 * Sorts SymbolsData by weight.
	 * @param symbols list of SymbolData, unsorted 
	 * @return list of SymbolData, sorted by weight
	 */
	protected Vector<SymbolData> sortSymbols(Vector<SymbolData> symbols){
		Vector<SymbolData> result = new Vector<SymbolData>();
		for(int i = 0;i<symbols.size();i++){
			int maxWeight = Integer.MIN_VALUE;
			int maxIndex = 0;
			for(int j =0;j<symbols.size();j++){
				if (symbols.get(j).getWeight()>maxWeight && !result.contains(symbols.get(j))){
					maxWeight = symbols.get(j).getWeight();
					maxIndex = j;
				}
			}
			result.add(symbols.get(maxIndex));
		}
		return result;
	}
	
	/**
	 * Returns all SymbolData of this rule, sorted by index (outgoing - incoming links).
	 * @return all SymbolData sorted by index
	 */
	public Vector<SymbolData> getIndexLHSSymbols() {
		return sortSymbolsIndex(getLHSSymbols());
	}
	
	protected int getSymbolIndex(EObject symbol){
		return rule.getLHS().getAllObjects().indexOf(symbol);
	}
	
	/**
	 * Sorts SymbolsData by index.
	 * @param symbols list of SymbolData, unsorted 
	 * @return list of SymbolData, sorted by index
	 */
	private Vector<SymbolData> sortSymbolsIndex(Vector<SymbolData> symbols){
		Vector<SymbolData> result = new Vector<SymbolData>();
		for(int i = 0;i<symbols.size();i++){
			int minIndex = Integer.MAX_VALUE;
			int symbolIndex = 0;
			for(int j =0;j<symbols.size();j++){
				if (symbols.get(j).getIndex()<minIndex && !result.contains(symbols.get(j))){
					minIndex = symbols.get(j).getIndex();
					symbolIndex = j;
				}
			}
			result.add(symbols.get(symbolIndex));
		}
		return result;
	}

	public Vector<InputData> getInputObjects() {
		return inputObjects;
	}

	public void setInputObjects(Vector<InputData> inputObjects) {
		this.inputObjects = inputObjects;
	}

}

/**
 * $Log: RuleData.java,v $
 * Revision 1.1.4.2  2006/12/15 14:21:23  reneschm
 * merged with head
 *
 * Revision 1.5  2006/11/01 13:12:21  enrico
 * *** empty log message ***
 *
 * Revision 1.4  2006/10/25 13:09:36  enrico
 * added Input Objects
 *
 * Revision 1.3  2006/10/24 12:57:55  bunjip
 * sortSymbols public->protected
 *
 * Revision 1.2  2006/10/09 11:40:13  bunjip
 * sortSymbols private => public
 *
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.3  2006/06/29 10:17:06  bunjip
 * Problem der impliziten Links gefixt
 *
 * Revision 1.2  2006/06/22 10:33:59  bunjip
 * initVariables() integriert
 *
 * Revision 1.1  2006/06/04 11:18:24  bunjip
 * Data-Klassen (zeitweise) wieder in Generator-Projekt integriert
 *
 * Revision 1.12  2006/06/03 16:11:36  bunjip
 * Fehler anhand von Tests beseitigt
 *
 * Revision 1.11  2006/06/03 15:23:02  bunjip
 * Fehler anhand von Tests beseitigt
 *
 * Revision 1.10  2006/05/31 21:05:28  bunjip
 * Anpassungen an neues Modell
 *
 * Revision 1.9  2006/05/30 21:06:32  bunjip
 * Anpassungen an neues Modell (nur offensichtliche)
 *
 * Revision 1.8  2006/05/24 12:18:54  bunjip
 * Initialisierung fuer alle Attribute
 *
 * Revision 1.7  2006/05/23 23:35:12  bunjip
 * mehrere EObjects pro SymbolData
 *
 * Revision 1.6  2006/05/23 13:48:52  bunjip
 * *** empty log message ***
 *
 * Revision 1.5  2006/05/23 13:00:11  bunjip
 * Warnings beseitigt und kleinere Verbesserungen
 *
 * Revision 1.4  2006/05/23 12:35:28  bunjip
 * vorlaeufige Version (keine Syntaxfehler)
 *
 * Revision 1.3  2006/05/22 21:11:15  bunjip
 * *** empty log message ***
 *
 * Revision 1.2  2006/05/22 14:29:54  bunjip
 * offensichtliche und leicht zu behebende Fehler beseitigt
 *
 * Revision 1.1  2006/05/15 09:09:41  enrico
 * *** empty log message ***
 *
 * Revision 1.5  2006/04/18 13:17:56  bunjip
 * Sortierung der (LHS-)Symbols
 *
 * Revision 1.4  2006/04/13 20:32:22  bunjip
 * javadoc hinzugefuegt
 *
 * Revision 1.3  2006/04/03 12:33:44  bunjip
 * Umstellung Modell
 *
 * Revision 1.2  2006/03/27 12:45:18  bunjip
 * layer hinzugefuegt
 *
 * Revision 1.1  2006/03/15 09:42:47  enrico
 * *** empty log message ***
 *
 * Revision 1.1  2006/03/14 17:06:10  eduardw
 * *** empty log message ***
 *
 * Revision 1.26  2006/03/06 10:13:14  bunjip
 * kleinere Fehler aufgrund NAC-Tests beseitigt
 *
 * Revision 1.25  2006/03/02 23:45:01  bunjip
 * unnoetige Importe entfernt und Vectoren parametrisiert
 *
 * Revision 1.24  2006/03/01 18:42:38  bunjip
 * ueberfluessige Attribute und Methoden geloescht
 *
 * Revision 1.23  2006/03/01 15:33:46  bunjip
 * Ueberfluessige oder doppelte Methoden entfernt
 *
 * Revision 1.22  2006/02/28 16:57:16  bunjip
 * Initialisierung der NAC-Queries
 *
 * Revision 1.21  2006/02/27 15:32:48  bunjip
 * getNACLinks(int) hinzugefuegt
 *
 * Revision 1.20  2006/02/24 17:09:28  bunjip
 * Veraenderung von SymbolData (implizite Darstellung) und abhaengiger Klassen - einfach gestestet und funktional
 *
 * Revision 1.19  2006/02/22 13:30:03  bunjip
 * kleinere Fehler beseitigt
 *
 * Revision 1.18  2006/02/20 13:58:27  bunjip
 * Anpassungen an das neue Regelmodell, erste oberflaechliche Tests erfolgreich
 *
 * Revision 1.17  2006/02/06 22:08:11  bunjip
 * Code fuer InjectivitaetsQuery hinzugefuegt (noch ungetestet)
 *
 * Revision 1.16  2006/02/02 13:20:32  bunjip
 * Gewichtung der Symbols anhand ausgehender bzw eingehender Kanten - getWeight() bzw getWeightedIndex(symbol)
 *
 * Revision 1.15  2006/01/30 14:15:14  enrico
 * neue Methoden fuer Strukturzugriffe
 *
 * Revision 1.14  2006/01/19 13:17:02  enrico
 * LHS und RHS Attribute hinzugefuegt
 *
 * Revision 1.13  2006/01/18 13:11:33  bunjip
 * Typ der veraenderten Attribute gespeichert
 *
 * Revision 1.12  2006/01/16 16:09:47  bunjip
 * Initialisierung der Attributaenderungen korrigiert
 *
 * Revision 1.11  2006/01/16 15:31:33  bunjip
 * Unterstuetzung fuer AttributChange erweitert
 *
 * Revision 1.10  2006/01/12 12:41:49  bunjip
 * getClassName hinzugefuegt
 *
 * Revision 1.9  2006/01/12 12:25:20  enrico
 * getLHSLinks hinzugefuegt
 *
 * Revision 1.8  2006/01/11 18:26:34  bunjip
 * Initialisierung von VariableData geaendert, beidseitige Referenzen zwischen Attributen und Variablen hergestellt
 *
 * Revision 1.7  2006/01/11 12:34:30  bunjip
 * Initialisierung der AttributChanges
 *
 * Revision 1.6  2006/01/06 20:29:03  bunjip
 * Abfrage fuer Regelarten hinzugefuegt
 *
 * Revision 1.5  2006/01/05 12:23:03  bunjip
 * getVarByIndex implementiert - liefert Namen der Variablen fuer ein SymbolData mit dem entsprechenden Index
 *
 */