package de.tuberlin.emt.generator.data;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * Class : SymbolData <br/>
 * Package : de.tuberlin.emfrule.generator.util <br/>
 * Project : de.tuberlin.emfrule.generator <br/>
 * Created On : 28.12.2005 <br/>
 * Description : Wrapper class to extract information from a
 * symbol element<br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */

public class SymbolData {
	private EObject symbol;
	private Vector<EObject> images = new Vector<EObject>();
	private RuleData rule;
	private String variableName;
	private boolean lhs = false;
	private boolean rhs = false;
	private String[] nacTypes;
	
	private Vector<LinkData> outgoing = new Vector<LinkData>();
	private Vector<LinkData> incoming = new Vector<LinkData>();
	private Vector<AttributeData> attributes = new Vector<AttributeData>();
		
	/**
	 * Constructor.
	 * @param symbol symbol that is represented by this SymbolData
	 * @param rule rule that contains this symbol
	 * @param index index of the NAC that contains this Symbol, or -1 otherwise
	 */
	public SymbolData(EObject symbol, RuleData rule, int index){
		this.nacTypes = new String[rule.getNACCount()];
		this.symbol = symbol;
		this.rule = rule;
		this.lhs = false;
		if(index==-1){
			this.rhs = true;
		}
		if(index==-2){
			this.lhs = true;
		}
		if(index>=0){
			this.nacTypes[index] = symbol.eClass().getName();
		}
		this.initLinks(symbol, index);
	}
	
	/**
	 * Creates a name for the variable that represents this symbol in the
	 * generated rule class. 
	 * @param counter number of this symbol within symbols of the same type
	 * @param isNew if symbol is created by the rule
	 */
	public void createVariableName(int counter, boolean isNew) {
		if (isNew)
			variableName = new String("new"+symbol.eClass().getName()+counter);
		else
			variableName = new String(symbol.eClass().getName().toLowerCase()+counter);
	}
	
	private EList getRHSAttributes(EObject origin, EObject image) {
		EList result = image.eClass().getEAllAttributes();
		if(origin.eClass() == image.eClass()) {
			for (int i=result.size()-1;i>=0;i--){
				EAttribute att = (EAttribute) result.get(i);
				if (origin.eGet(att).equals(image.eGet(att))){
					result.remove(att);
				}
			}
		}
		return result;
	}
	
	/**
	 * Initialises attributes, that belong to this symbol.
	 * @param sym symbol that is represented
	 * @param index index of NAC, that contains the symbol or -1 otherwise
	 */
	protected void addAttribute(AttributeData att){
		this.attributes.add(att);
	}
	
	
	/**
	 * Initialises all outgoing links of this symbol.
	 * @param sym symbol that is represented 
	 * @param index index of NAC, that contains the symbol or -1 otherwise
	 */
	protected void initLinks(EObject sym, int index){
		EList refs = sym.eClass().getEAllReferences();
		for(int i=0; i<refs.size();i++){
			EReference current = (EReference)refs.get(i);
			if(sym.eIsSet(current)){
				if(!current.isMany()){
					LinkData currentData = new LinkData(current, this,this.rule,(EObject)sym.eGet(current),index);
					outgoing.add(currentData);
					rule.addLink(currentData);
				}
				else{
					EList list = (EList)sym.eGet(current); 
					for(int j=0; j<list.size();j++){
						EObject target = (EObject)list.get(j);
						LinkData currentData = new LinkData(current, this,this.rule,target,index);
						outgoing.add(currentData);
						rule.addLink(currentData);
					}	
				}
			}
		}
	}
	
	/**
	 * Adds a link to the incoming links.
	 * @param link link that is added to incoming links
	 */
	protected void addIncoming(LinkData link){
		this.incoming.add(link);
	}
	
	protected void removeOutgoing(LinkData link){
		this.outgoing.remove(link);
	}
	
	
	/**
	 * Adds mapped symbol to the symbols that are represented by this.
	 * @param data mapped symbol, represented by this object
	 */
	protected void addImage(EObject data, int index){
		if(index == -1) {
			this.images.add(0,data);
			this.rhs = true;
		}
		else {
			this.images.add(data);
			this.nacTypes[index] = data.eClass().getName();
		}
		this.initLinks(data,index);
	}
	
	protected EObject getLHSObject(){
		if(rule.isLHSObject(symbol)) return symbol;
		return null;
	}
	
	protected EObject getNACObject(int index){
		if(rule.isNACObject(symbol,index)) return symbol;
		for(int i=0; i<images.size(); i++){
			if(rule.isNACObject(images.get(i),index)) return images.get(i);
		}
		return null;
	}
	
	protected EObject getRHSObject(){
		if(rule.isRHSObject(symbol)) return symbol;
		if(images.size()>0 && rule.isRHSObject(images.get(0))) 
			return images.get(0);
		return null;
	}
	
	/**
	 * Returns index of the symbol inside the rule.
	 * @return index of symbol inside of the rule.
	 */
	public int getIndex(){
		return rule.getSymbolIndex(this.symbol);
	}
	
	/**
	 * Returns type of symbol.
	 * @return type of symbol
	 */
	public String getType(){
		return symbol.eClass().getName();
	}
	
	/**
	 * Type of this symbol in the NAC with the index.
	 * @param index index of the NAC
	 * @return type of the symbol in this NAC
	 */
	public String getNACType(int index){
		return nacTypes[index];
	}
	
	/**
	 * Returns variable name for this symbol in the generated rule class.
	 * @return variable name for this symbol in the generated rule class
	 */
	public String getVariableName(){
		return variableName;
	}
	
	/**
	 * Returns variable name for this symbol in the generated rule class,
	 * first character in upper case.
	 * @return variable name for this symbol in the generated rule class,
	 * first character in upper case
	 */
	public String getUpVariableName(){
		String result = variableName;
		result = result.substring(0,1).toUpperCase().concat(result.substring(1));
		return result;
	}
	
	/**
	 * Returns all outgoing links for this symbol in a side of the rule.
	 * @param side abbreviation for the side, l for LHS, r for RHS or NAC index
	 * @return all outgoing links, that occur in the specified rule side  
	 */
	public Vector<LinkData> getOutgoing(){
		return this.outgoing;
	}
	
	public Vector<LinkData> getIncoming(){
		return this.incoming;
	}

	/**
	 * Returns all incoming links for this symbol in a side of the rule.
	 * @param side abbreviation for the side, l for LHS, r for RHS or NAC index
	 * @return all incoming links, that occur in the specified rule side  
	 */
	public Vector<LinkData> getIncoming(String side){
		Vector<LinkData> result = new Vector <LinkData>();
		for(Iterator it = incoming.iterator();it.hasNext();){
			LinkData data = (LinkData)it.next();
			if(side.equals("l")){
				if(data.isLHS()) result.add(data);
			}else if(side.equals("r")){
				if(data.isRHS()) result.add(data);
			}else 
				if(data.isNAC(Integer.parseInt(side))){
					result.add(data);
			}
		}
		return result;
	}
	
	/**
	 * Returns all outgoing links for this symbol in a side of the rule.
	 * @param side abbreviation for the side, l for LHS, r for RHS or NAC index
	 * @return all outgoing links, that occur in the specified rule side  
	 */
	public Vector<LinkData> getOutgoing(String side){
		Vector<LinkData> result = new Vector <LinkData>();
		for(Iterator it = outgoing.iterator();it.hasNext();){
			LinkData data = (LinkData)it.next();
			if(side.equals("l")){
				if(data.isLHS()) result.add(data);
			}else if(side.equals("r")){
				if(data.isRHS()) result.add(data);
			}else 
				if(data.isNAC(Integer.parseInt(side))){
					result.add(data);
			}
		}
		return result;
	}
	
	
	/**
	 * Returns if object is created without containment in EMF tree structure.
	 * @return if object is created without containment in EMF tree structure
	 */
	public boolean isConsistent() {
		if (isLHS())
			return true;
		else {
			Vector<LinkData> incoming = getIncoming("r");
			for (int i = 0; i < incoming.size(); i++)
				if(incoming.get(i).isContainment())
					return true;
			EClass eclass = symbol.eClass();
			EList classifiers = eclass.getEPackage().getEClassifiers();
			for (int i=0; i<classifiers.size(); i++) {
				if (!(classifiers.get(i) instanceof EClass)) continue;
				EClass current = (EClass) classifiers.get(i);
				if (current==eclass || current.isSuperTypeOf(eclass) || eclass.isSuperTypeOf(current)) continue;
				for (int j=0; j<current.getEAllContainments().size(); j++) {
					EReference containment = (EReference) current.getEAllContainments().get(j);
					if (containment.getEContainingClass() != current) continue;
					if (containment.getEReferenceType()==eclass) return false; 
					if (containment.getEReferenceType().isSuperTypeOf(eclass)) return false;
				}
			}
			return true;
		}
	}
	
	/**
	 * Returns all attributes for this symbol in a side of the rule.
	 * @param side abbreviation for the side, l for LHS, r for RHS or NAC index
	 * @return all attributes for this symbol in the specified side of the rule 
	 */
	public Vector<AttributeData> getAttributes(String side){
		Vector<AttributeData> result = new Vector <AttributeData>();
		for(Iterator it = attributes.iterator();it.hasNext();){
			AttributeData data = (AttributeData)it.next();
			if(side.equals("l")){
				if(data.isLHS()) result.add(data);
			}else if(side.equals("r")){
				if(data.isRHS()) result.add(data);
			}else 
				if(data.isNAC(Integer.parseInt(side))){
					result.add(data);
			}
		}
		return result;
	}
	
	/**
	 * Returns all attributes of this symbol and mapped symbols.
	 * @return all attributes of this symbol and mapped symbols
	 */
	public Vector<AttributeData> getAttributes(){
		return this.attributes;
	}
		
	/**
	 * Returns rule that contains this symbol.
	 * @return rule that contains this symbol
	 */
	public RuleData getRule(){
		return this.rule;
	}
	
	/**
	 * Returns if represented symbol is created by the rule.
	 * @return if represented symbol is created by the rule
	 */
	public boolean isCreated(){
		if(!isLHS() && isRHS()){
			return true;
		}
		return false;
	}
		
	/**
	 * Returns if represented symbol is created by the rule.
	 * @return if represented symbol is created by the rule
	 */
	public boolean isDeleted(){
		if(isLHS()&& !isRHS()){
			return true;
		}
		return false;
	}
	
	/**
	 * Returns if represented symbol occurs in LHS.
	 * @return if represented symbol occurs in LHS
	 */
	public boolean isLHS(){
		return this.lhs;
	}
	
	/**
	 * Returns if represented symbol occurs in RHS.
	 * @return if represented symbol occurs in RHS
	 */
	public boolean isRHS(){
		return this.rhs;
	}
		
	/**
	 * Returns if represented symbol occurs in a NAC.
	 * @return if represented symbol occurs in a NAC
	 */
	public boolean isNAC(){
		if(rule.isNACObject(symbol)) return true;
		for(int i=0; i<images.size();i++){
			if(rule.isNACObject(images.get(i))) return true;
		}
		return false;
	}
	
	/**
	 * Returns if represented symbol occurs in the NAC with a given index.
	 * @param i index of the NAC
	 * @return if represented symbol occurs in the NAC with specified index
	 */
	public boolean isNAC(int i){
		if(nacTypes[i]!=null) return true;
		return false;
	}
	
	/**
	 * Returns weight of the symbol, which is: number of outgoing - number of
	 * incoming links in the LHS. 
	 * @return weight of the symbol
	 */
	public int getWeight(){
		return getOutgoing().size()-getIncoming().size();
	}

	public EObject getSymbol() {
		return symbol;
	}
	
}
/**
 * $Log: SymbolData.java,v $
 * Revision 1.1.4.2  2006/12/15 14:21:23  reneschm
 * merged with head
 *
 * Revision 1.3  2006/10/25 13:09:36  enrico
 * added Input Objects
 *
 * Revision 1.2  2006/10/04 14:05:47  bunjip
 * isConsistent adapted to rootContainers
 *
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.4  2006/06/29 10:17:06  bunjip
 * Problem der impliziten Links gefixt
 *
 * Revision 1.3  2006/06/19 12:15:13  bunjip
 * Setzen von nacTypes im Konstruktor
 *
 * Revision 1.2  2006/06/05 11:21:03  bunjip
 * Fehler behoben
 *
 * Revision 1.1  2006/06/04 11:18:24  bunjip
 * Data-Klassen (zeitweise) wieder in Generator-Projekt integriert
 *
 * Revision 1.10  2006/05/31 21:05:28  bunjip
 * Anpassungen an neues Modell
 *
 * Revision 1.9  2006/05/29 15:18:41  bunjip
 * fuer RHS nur geanderte Attribute betrachten
 *
 * Revision 1.8  2006/05/24 12:18:54  bunjip
 * Initialisierung fuer alle Attribute
 *
 * Revision 1.7  2006/05/23 23:35:12  bunjip
 * mehrere EObjects pro SymbolData
 *
 * Revision 1.6  2006/05/23 13:51:49  bunjip
 * *** empty log message ***
 *
 * Revision 1.5  2006/05/23 13:48:52  bunjip
 * *** empty log message ***
 *
 * Revision 1.4  2006/05/23 13:00:11  bunjip
 * Warnings beseitigt und kleinere Verbesserungen
 *
 * Revision 1.3  2006/05/22 21:11:15  bunjip
 * *** empty log message ***
 *
 * Revision 1.2  2006/05/22 14:29:54  bunjip
 * offensichtliche und leicht zu behebende Fehler beseitigt
 *
 * Revision 1.1  2006/05/15 09:09:41  enrico
 * *** empty log message ***
 *
 * Revision 1.4  2006/04/13 22:12:20  bunjip
 * javadoc hinzugefuegt
 *
 */