package de.tuberlin.emt.generator.data;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.emf.ecore.EPackage;

import de.tuberlin.emt.model.Rule;
import de.tuberlin.emt.model.Transformation;

/**
 * Class : TransformationData <br/>
 * Package : de.tuberlin.emfrule.generator.util <br/>
 * Project : de.tuberlin.emfrule.generator <br/>
 * Created On : 20.12.2005 <br/>
 * Description : Wrapper class to extract information from a
 * transformation model<br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */
public class TransformationData {
	private Transformation transformation;
	private Vector<RuleData> rules;
	private int ruleIndex;
	//name of the package that contains the generated code
	private String pack; 
	//name of the package that contains model code
	private String model;
	private Vector<String> modelURI;
	
	/**
	 * Constructor.
	 * @param transformation transformation that is represented by this TransformationData
	 */
	public TransformationData(Transformation transformation){
		this.transformation = transformation;
		this.modelURI = new Vector<String>();
		this.initRules();
		for (Iterator it = transformation.getPackages().iterator(); it.hasNext();) {
			EPackage ePackage = (EPackage) it.next();
			modelURI.add(ePackage.getNsURI());
		}
	}
	
	/**
	 * Returns name of the package that will contain the rule classes.
	 * @return name of the package that will contain rule classes
	 */
	public String getPackageName(){
		//return new String(this.transformation.getSourceModel());
		return this.pack;
	}
	
	/**
	 * Sets the name of the package.
	 * @param pack new name of the package
	 */
	public void setPackageName(String pack){
		this.pack = pack;
	}
	
	/**
	 * Returns name of the model project, to which the rules will be applied.
	 * @return name of the model project
	 */
	public String getModelName(){
		return this.model;
	}
	
	/**
	 * Sets name of the model project.
	 * @param model new name of model project
	 */
	public void setModelName(String model){
		this.model = model;
	}
	
	
	/**
	 * Returns name of the plugin project, that contains the created classes.
	 * @return name of the plugin project
	 */
	public String getPluginName(){
		return new String("TestPlugin");
	}
	
	/**
	 * Returns RuleData, which has the given name.
	 * @param name name of a rule
	 * @return RuleData with this name
	 */
	public RuleData getRuleByName(String name){
		for(Iterator it = rules.iterator();it.hasNext();){
			RuleData current = (RuleData)it.next();
			if(current.getName().equals(name)){
				return current;
			}
		}
		return null;
	}
	
	/**
	 * Returns rule with given index.
	 * @param index index of the rule
	 * @return rule with this index
	 */
	public RuleData getRuleByIndex(int index){
		return this.rules.get(index);
	}
	
	/**
	 * Initialises the rules of this transformation.
	 */
	protected void initRules(){
		this.rules = new Vector<RuleData>();
		for(Iterator it = this.transformation.getRules().iterator();it.hasNext();){
			Rule current = (Rule)it.next();
			this.rules.add(new RuleData(current));
		}
	}
	
	/**
	 * Returns ruleIndex - the index of the current rule.
	 * @return index of the current rule
	 */
	public int getRuleIndex(){
		return this.ruleIndex;
	}
	
	/**
	 * Sets new ruleIndex - new current rule. 
	 * @param index index of the new current rule
	 */
	public void setRuleIndex(int index){
		this.ruleIndex = index;
	}
	
	/**
	 * Returns current rule.
	 * @return current rule
	 */
	public RuleData getRule(){
		return this.rules.get(this.ruleIndex);
	}
	
	/**
	 * Returns number of rules in transformation.
	 * @return nomber of rules in transformation
	 */
	public int getRuleCount(){
		return this.rules.size();
	}
	
	/**
	 * Returns index of the highest layer.
	 * @return index of highest layer
	 */
	public int getMaxLayer(){
		int value = 0;
		for(int i = 0; i<rules.size();i++){
			if(rules.get(i).getLayer()>value){
				value = rules.get(i).getLayer();
			}
		}
		return value;
	}
	
	/**
	 * Returns all rules inside a layer.
	 * @param layer index of the layer
	 * @return all rules inside this layer
	 */
	public Vector<RuleData> getLayerRule(int layer){
		Vector<RuleData> value = new Vector<RuleData>();
		for(int i=0; i<rules.size();i++){
			if(rules.get(i).getLayer()==layer) value.add(rules.get(i));
		}
		return value;
	}
	
	/**
	 * Returns rule with specified index inside a layer.
	 * @param layer index of layer
	 * @param index index of rule iside of the layer
	 * @return rule from the layer and with specified index
	 */
	public RuleData getLayerRule(int layer, int index){
		Vector<RuleData> layerRules = getLayerRule(layer);
		return layerRules.get(index);
	}
	/**
	 * @return the transformation
	 */
	public Transformation getTransformation() {
		return transformation;
	}

	public Vector<String> getModelURI() {
		return modelURI;
	}

	public void setModelURI(Vector<String> modelURI) {
		this.modelURI = modelURI;
	}
	
}

