package de.tuberlin.emt.generator.data;

import java.util.Vector;

import de.tuberlin.emt.common.util.TypeConverter;
import de.tuberlin.emt.model.Variable;



/**
 * Class : AttributeData <br/>
 * Package : de.tuberlin.emfrule.generator.util <br/>
 * Project : de.tuberlin.emfrule.generator <br/>
 * Created On : 29.12.2005 <br/>
 * Description : Wrapper class to extract information from a
 * transformation variable<br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */

public class VariableData{//scheinbar OK mit neuem Modell
	private Variable var;
	private RuleData rule;
	private Vector<AttributeData> atts = new Vector<AttributeData>(0);
	
	/**
	 * Constructor.
	 * @param var Variable that is represented by this VariableData
	 * @param rule RuleData that contains this variable
	 */
	public VariableData(Variable var, RuleData rule){
		this.var = var;
		this.rule = rule;
	}
	
	/**
	 * Adds a reference to a attribute that is connected with this variable.
	 * @param att attribute that is connected with this variable
	 */
	protected void addAttribute(AttributeData att){
		this.atts.add(att);
	}
	
	/**
	 * Returns if this variable is a input parameter of the rule.
	 * @return if variable is input parameter of the rule
	 */
	public boolean isInput(){
		return var.isInput();
	}
	
	/**
	 * Returns name of the variable.
	 * @return name of the variable
	 */
	public String getName(){
		return var.getName();
	}
	
	/**
	 * Returns name of the variable, first character in upper case.
	 * @return name of the variable, first character in upper case
	 */
	public String getUpName(){
		String result = var.getName();
		result = result.substring(0,1).toUpperCase().concat(result.substring(1));
		return result;
	}
	
	/**
	 * Returns type of the variable.
	 * @return type of the variable
	 */
	public String getType(){
		return this.var.getType().getName();
	}
	
	public String getJavaType(){
		return TypeConverter.toJavaClass(this.var.getType().getName());
	}
	
	/**
	 * Returns rule that contains this variable.
	 * @return rule that contains this variable
	 */
	public RuleData getRule(){
		return this.rule;
	}
	
	/**
	 * Returns all attributes that are connected with this variable.
	 * @return all attributes that are connected with this variable
	 */
	public Vector<AttributeData> getAtts(){
		return this.atts;
	}
	
	
	/**
	 * Get all occurences of this Variable in a certain part of the Rule.
	 * @param side part of the Rule - either NAC index (side>=0) or LHS (side<0)
	 * @return all occurences of this Variable in the specified part
	 */
	public Vector<AttributeData> getAtts(int side){
		Vector<AttributeData> result = new Vector<AttributeData>();
		AttributeData lhsAtt = null;
		boolean hasLHS = false;
		for(int i=0; i<atts.size();i++){
			AttributeData current = atts.get(i);
			if(current.isLHS() && lhsAtt == null){
				lhsAtt = current;
			}
			if(current.isLHS() && side<0){
				result.add(current);
			}
			if(current.isNAC(side) && side>=0){
				result.add(current);
				if(current.getOwner().isLHS()) hasLHS = true;
			}
		}
		
		//side = LHS: sort SymbolData by weight
		if(side<0 && result.size()>2){
			Vector<SymbolData> lhsSym = new Vector<SymbolData>();
			for(int j=0; j<result.size();j++){
				lhsSym.add(result.get(j).getOwner());
			}
			Vector<SymbolData> sortSym = rule.sortSymbols(lhsSym);
			Vector<AttributeData> sortResult = new Vector<AttributeData>();
			for(int j=0; j<sortSym.size();j++){
				SymbolData sym = sortSym.get(j);
				for(int k=0; k<result.size();k++){
					if(result.get(k).getOwner().equals(sym)){
						sortResult.add(result.get(k));
					}
				}
			}
			return sortResult; 	
		}
		
		//side = NAC: if variable already occurs in LHS, but there is no LHS object
		//that includes this variable inside the NAC, get value from arbitrary LHS object    
		if(side>=0 && lhsAtt!=null && !hasLHS){
			result.add(lhsAtt);
		}
		
		return result;
	}
	
	
	
}
/**
 * $Log: VariableData.java,v $
 * Revision 1.1.4.2  2006/12/15 14:21:23  reneschm
 * merged with head
 *
 * Revision 1.4  2006/10/25 13:02:19  bunjip
 * getAtts(int) changed and documented
 *
 * Revision 1.3  2006/10/24 12:59:46  bunjip
 * getAtts(int) added
 *
 * Revision 1.2  2006/10/10 10:05:49  enrico
 * toJava->toJavaClass
 *
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.2  2006/06/06 17:59:36  bunjip
 * Umwandlung EMF->JavaTyp
 *
 * Revision 1.1  2006/06/04 11:18:23  bunjip
 * Data-Klassen (zeitweise) wieder in Generator-Projekt integriert
 *
 * Revision 1.3  2006/05/30 21:06:32  bunjip
 * Anpassungen an neues Modell (nur offensichtliche)
 *
 * Revision 1.2  2006/05/22 14:29:54  bunjip
 * offensichtliche und leicht zu behebende Fehler beseitigt
 *
 * Revision 1.1  2006/05/15 09:09:41  enrico
 * *** empty log message ***
 *
 * Revision 1.2  2006/04/13 21:02:17  bunjip
 * javadoc hinzugefuegt
 *
 */