package de.tuberlin.emt.generator.jet;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.emf.codegen.merge.java.JControlModel;
import org.eclipse.emf.codegen.merge.java.JMerger;
import org.eclipse.emf.codegen.util.CodeGenUtil;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.osgi.util.ManifestElement;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import org.osgi.framework.Constants;

import de.tuberlin.emt.generator.GeneratorPlugin;


/**
 * Class : JETGateway <br/>
 * Package : de.tuberlin.emftransformation.generator.jet <br/>
 * Project : de.tuberlin.emftransformation.generator <br/>
 * Created On : 20.12.2005 <br/>
 * Description : This class represents a bridge between the data structure classes an the JETEmitter.<br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */
public class JETGateway {

	
	/**
	 * Emitter for the target Files.
	 */
	private JETEmitter emitter;
	
	private boolean isSourceCode;

	/**
	 * The target folder in which the generated java - File.
	 */
	private String targetProject;

	/**
	 * The package in which the generated java file is saved.
	 */
	private String packagename = "";

	/**
	 * The name of the generated java - file.
	 */
	private String targetFile = "";

	/**
	 * The Path of the template for the java - file.
	 */
	private String templateUri = "";

	private String templateFileExtension = "javajet";
	
	/**
	 * A list of *.jar files which are needed to compile the translated template files.
	 */
	private List<IClasspathEntry> classpathEntries = new Vector<IClasspathEntry>();

	/**
	 * Creates a bridge between the Generator main class and the JETEmitter. The targetFolder in which
	 * all generated files are put is "src". Furthermore all libraries are added to a list which are
	 * needed to compile the translated template files in the <em>.JETEmitters</em> project. 
	 * @param targetFolder 
	 */
	public JETGateway(String targetProject) {
		this.targetProject = targetProject;
		
		try {
			setAllLibraries(GeneratorPlugin.ID);
			
		} catch (JETException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Invokes the JET template specified in the <code>data</code> with the
	 * model specified in the <code>data</code> and returns the generated
	 * text as a String.
	 * <p>
	 * This implementation uses a <code>JETEmitter</code> to translate the
	 * template to a Java implementation class. The translated class is created
	 * in a hidden project called <code>.JETEmitters</code>.
	 * <p>
	 * In order to be able to compile the translated template implementation
	 * class, the classes used by the model specified in the <code>data</code>
	 * must be available in the classpath. For this reason, this method sets the
	 * first runtime library of the plugin specified in the <code>Config</code>
	 * as a classpath variable to the <code>.JETEmitters</code> project.
	 * 
	 * @param monitor
	 *            the progress monitor to use. 
	 * @param data 
	 * @return the source code text generated by the JET template
	 * @throws JETException 
	 */
	public String generate(IProgressMonitor monitor, Object data)
			throws CoreException {
		ClassLoader classLoader = getClass().getClassLoader();                                                                                           
		emitter = new JETEmitter(getTemplateUri(), classLoader);
		emitter.getClasspathEntries().addAll(classpathEntries);
		IProgressMonitor sub = new SubProgressMonitor(monitor, 3);

		String result="";
		try {
			result = emitter.generate(sub, new Object[] { data });
			save(monitor, result);
			monitor.worked(1);
		} catch (JETException e) {
			e.printStackTrace();
		}
		
		return result;

	}

	/**
	 * Sets all libraries to the classpath that are needed to compile the translated 
	 * template files.
	 * @param pluginID The ID of the TIGERPlugin.
	 * @throws JETException
	 */
	private void setAllLibraries(String pluginID) throws JETException {

		String[] variableNames = { "EMFTRANSFORMATION_JAR" };
		Bundle bundle = Platform.getBundle(pluginID);
		String requires = (String) bundle.getHeaders().get(
				Constants.BUNDLE_CLASSPATH);
		ManifestElement[] elements = null;
		try {
			elements = ManifestElement.parseHeader(Constants.BUNDLE_CLASSPATH,
					requires);
		} catch (BundleException e) {
			e.printStackTrace();
		}
		for (int i = 0; i < variableNames.length; ++i) {
			ManifestElement element = elements[i];
			
				URL url = bundle.getEntry(element.getValue());
				if(url != null){
					try {
				
				IPath path = new Path(FileLocator.toFileURL(url).getFile());
				if (!path.equals(JavaCore
						.getClasspathVariable(variableNames[i]))) {
					JavaCore.setClasspathVariable(variableNames[i], path, null);
				}
			} catch (MalformedURLException exception) {
				throw new JETException(exception);
			} catch (JavaModelException exception) {
				throw new JETException(exception);
			} catch (IOException exception) {
				throw new JETException(exception);
			}
			
			IClasspathEntry entry = JavaCore.newVariableEntry(new Path(
					variableNames[i]), null, null);
			classpathEntries.add(entry);
			}
		}
		String t = Platform.getInstallLocation().getURL().getPath();
		classpathEntries.add(JavaCore.newContainerEntry(new Path("org.eclipse.pde.core.requiredPlugins")));
		String version = getPluginVersion("org.eclipse.osgi");
		classpathEntries.add(JavaCore.newLibraryEntry(new Path(t +"plugins/org.eclipse.osgi_"+ version +".jar"),null,null));
		version = getPluginVersion("org.eclipse.osgi.services");
		classpathEntries.add(JavaCore.newLibraryEntry(new Path(t +"plugins/org.eclipse.osgi.services_"+ version +".jar"),null,null));
		version = getPluginVersion("org.eclipse.core.runtime");
		classpathEntries.add(JavaCore.newLibraryEntry(new Path(t +"plugins/org.eclipse.core.runtime_"+ version +".jar"),null,null));
		version = getPluginVersion("org.eclipse.osgi.util");
		classpathEntries.add(JavaCore.newLibraryEntry(new Path(t +"plugins/org.eclipse.osgi.util_"+ version +".jar"),null,null));
		//IClasspathEntry entry = JavaCore.newVariableEntry(new Path(t +"workspace/de.tuberlin.emt.common/emtcommon.jar"),null,null);
		//classpathEntries.add(entry);
	}
	
	private String getPluginVersion(String id){
		Bundle b =  Platform.getBundle(id);
		return(String) b.getHeaders().get("Bundle-Version");
	}

	/**
	 * Saves the specified contents to a location specified by the
	 * <code>data</code> settings. The location of the file to save is found
	 * by finding or creating the container (folder) for the <code>data</code>
	 * 's package in the <code>data</code>'s target folder. The name of the
	 * file to save is the <code>data</code>'s target file.
	 * 
	 * @param monitor
	 *            the progress monitor to use. 
	 * @param contents
	 *            the byte contents of the file to save
	 * @param shouldBeMerged 
	 * @return IFile object
	 * @throws CoreException
	 * @throws JETException 
	 */
	public IFile save(IProgressMonitor monitor, String contents)
			throws CoreException, JETException {

		IContainer container = findOrCreateContainer(monitor, targetProject,
				packagename);
		if (container == null) {
			throw new JETException(
					"Cound not find or create container for package "
							+ packagename + " in " + targetProject);
		}
		
		IFile result = container.getFile(new Path(this.targetFile));
		
			InputStream contents_res = new ByteArrayInputStream(contents.getBytes());
			if (!result.exists()) {
				result.create(contents_res, false, monitor);
			} else {
				container.refreshLocal(1, monitor);
				result.setContents(contents_res, true, false, monitor);
			}

		return result;
	}
	
	public void createJavaProject(IProgressMonitor progressMonitor) 
		throws CoreException{
		
		IProgressMonitor sub = new SubProgressMonitor(progressMonitor, 1);
		IPath projectPath = new Path(targetProject);
		IPath localLocation = null;
		
		IContainer container = CodeGenUtil.EclipseUtil.findOrCreateContainer(projectPath, true,
				localLocation, sub);
		IProject project = container.getProject();
		
		String[] natures = {"org.eclipse.pde.PluginNature", JavaCore.NATURE_ID};
		IProjectDescription desc = project.getDescription();
		desc.setNatureIds(natures);
		project.setDescription(desc,progressMonitor);
		IJavaProject javaProject = JavaCore.create(project);
		
		IPath path = project.getFullPath().append("bin");
		javaProject.setOutputLocation(path, null);
		ArrayList<IClasspathEntry> entris = new ArrayList<IClasspathEntry>();
		path = project.getFullPath().append("src");
		// set classpath for src-folder
		
		entris.add(JavaCore.newSourceEntry(path));
		entris.add(JavaCore.newContainerEntry(new Path("org.eclipse.pde.core.requiredPlugins")));
		entris.add(JavaCore.newContainerEntry(new Path(
		"org.eclipse.jdt.launching.JRE_CONTAINER")));
		
		IClasspathEntry[] arr = new IClasspathEntry[entris.size()];
		for(int i = 0; i < arr.length; i++){
			arr[i] = (IClasspathEntry) entris.get(i);
		}
		
		// add all libraries to the classpath
		javaProject.setRawClasspath( arr, null);
	}
	
	private IContainer findOrCreateContainer(IProgressMonitor progressMonitor,
			String targetDirectory, String packageName) throws CoreException {

		IPath outputPath = null;
		if(isSourceCode)
			outputPath = new Path(targetProject + "/src/" 
				+ packageName.replace('.', '/'));
		else
			outputPath = new Path(targetProject + packageName.replace('.', '/'));

		IProgressMonitor sub = new SubProgressMonitor(progressMonitor, 1);
		IPath localLocation = null; // use default
		IContainer container = CodeGenUtil.EclipseUtil.findOrCreateContainer(outputPath, true,
				localLocation, sub);

		return container;
	}
	
	/**
	 * @param string
	 */
	public void setTargetProject(String string) {
		this.targetProject = string;
	}
	
	public void setTargetFile(String string) {
		if (isSourceCode && !string.endsWith("java"))
			this.targetFile = string + ".java";
		else
			this.targetFile = string;
	}
	
	
	public void setIsCode(boolean value) {
		this.isSourceCode = value;
	}

	/**
	 * @return Returns the templateUri.
	 * 
	 */
	public String getTemplateUri() {
		return templateUri;
	}
	
	/**
	 * Creates and initializes a new JMerger
	 * 
	 * @return JMerger
	 */
	private JMerger getJMerger() {
		JMerger jmerger = new JMerger();
		String uri = getPluginPath() + "/src/tiger/generator/jet/merge.xml";
		JControlModel controlModel = new JControlModel();
		
		return jmerger;
	}

	public void setTemplateFileExtension(String extension) {
		templateFileExtension = extension;
	}
	
	
	/**
	 * Sets the path to the templates. 
	 * @param packagename The packagename inside the template folder.
	 * @param filename The filename of the template.
	 */
	public void setTemplateUri(String path) {
		StringBuffer buf = new StringBuffer();
		buf.append(getPluginPath());
		buf.append("/src/de/tuberlin/emt/generator/templates/");
		buf.append(path);
		buf.append("."+templateFileExtension);
		this.templateUri = buf.toString();
	}

	/**
	 * Returns the path of the TIGER Generator Project.
	 * @return The path of the TIGER Generator Project.
	 */
	public static String getPluginPath() {
		//Object reg=Platform.getPluginRegistry();
		URL pluginURL = GeneratorPlugin.getDefault().find(new Path("."));
		URL p = null;
		try {
		p = Platform.resolve(pluginURL);
		return p.getPath();
		}
		catch (IOException e) {}
		
		return "";
	}


	/**
	 * Sets the package in which the generated file should be stored. They are relative to
	 * the targetFolder.
	 * @param packagename
	 *            The packagename to set.
	 */
	public void setPackagename(String packagename) {
		this.packagename = packagename;
	}
	
	/**
	 * Returns the file to be generated.
	 * @return Returns the targetFile.
	 */
	public String getTargetFile() {
		return targetFile;
	}
}

/**
 * $Log: JETGateway.java,v $
 * Revision 1.1.4.1  2006/10/29 17:54:35  reneschm
 * added IOObjects merge with head
 *
 * Revision 1.2  2006/09/18 11:05:22  enrico
 * Queries aus rules nach queries package verschoben
 *
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.4  2006/06/04 11:17:31  bunjip
 * setAllLibraries wieder aktiviert um die jar-Datei in runtime-workspace zu integrieren
 *
 * Revision 1.3  2006/06/02 17:28:22  bunjip
 * Templates werden gefunden
 *
 * Revision 1.2  2006/05/24 13:11:51  bunjip
 * *** empty log message ***
 *
 * Revision 1.1  2006/05/22 13:43:03  enrico
 * *** empty log message ***
 *
 * Revision 1.5  2006/04/11 08:21:20  enrico
 * Progressmonitor hinzugefuegt
 *
 * Revision 1.4  2006/03/21 19:56:29  bunjip
 * Anpassungen an Ecore
 *
 * Revision 1.3  2006/03/21 13:48:10  eduardw
 * startgraph
 *
 * Revision 1.2  2006/03/20 01:26:59  bunjip
 * emfrule -> emftransformation
 *
 * Revision 1.1  2006/03/15 09:42:47  enrico
 * *** empty log message ***
 *
 * Revision 1.1  2006/03/14 17:06:10  eduardw
 * *** empty log message ***
 *
 * Revision 1.5  2006/01/19 15:46:22  enrico
 * Allgemeine Pfadermittlung
 *
 * Revision 1.4  2006/01/16 12:06:36  enrico
 * Interface fuer Dateigenerierung vereinheitlicht
 *
 * Revision 1.3  2006/01/11 13:52:08  enrico
 * Erzeugten Code zum Projekt erweitert
 *
 * Revision 1.2  2005/12/28 18:06:08  bunjip
 * ClassLoader als Parameter an den Emitter weitergegeben
 *
 * Revision 1.1  2005/12/20 15:33:59  enrico
 * initial version
 *
 */