package de.tuberlin.emt.generator.model;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import de.tuberlin.emt.model.EMTFactory;
import de.tuberlin.emt.model.EMTPackage;
import de.tuberlin.emt.model.Transformation;
import de.tuberlin.emt.model.impl.EMTPackageImpl;


/**
 * Class : TransformationModelManager <br/>
 * Package : de.tuberlin.emftransformation.generator.model <br/>
 * Project : de.tuberlin.emftransformation.generator <br/>
 * Created On : 20.12.2005 <br/>
 * Description : Drives the model. Acts as the model entry point.<br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */
public class TransformationModelManager {
	/**
	 * For the purpose of the simple editor, a file can only contain a transformation model.
	 * In EMF, a resource provides the way to have access to the model content.
	 */
	private Resource resource = null;

	/**
	 * Contains the factory associated with the model.
	 */
	private static EMTFactory transformationModelFactory = null;

	/**
	 * Gives access to the top level network contained in the resource.
	 */
	private Transformation transformation = null;

	/**
	 * Returns the resource containing the network. Uses lazy initialization.
	 * @param path
	 * @return
	 */
	public Resource getResource(IPath path) {
		if (resource == null) {
			ResourceSet resSet = getResourceSet();
			resource =
				resSet.getResource(
					URI.createPlatformResourceURI(path.toString()),
					true);
		}
		return resource;
	}

	/**
	 * Creates a resource to contain the transformation model. The resource file does not exist yet.
	 * @param path
	 * @return
	 */
	private Resource createResource(IPath path) {
		if (resource == null) {
			ResourceSet resSet = getResourceSet();
			resource =
				resSet.createResource(
					URI.createPlatformResourceURI(path.toString()));
		}
		return resource;
	}

	/**
	 * Returns the resource set.
	 * @param 
	 * @return
	 */
	private ResourceSet getResourceSet() {
		// Initialize the transformationmodel package
		EMTPackageImpl.init();
		// Register the XMI resource factory for the .tfm extension
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map m = reg.getExtensionToFactoryMap();
		m.put("emt", new XMIResourceFactoryImpl());
		// Obtain a new resource set
		return new ResourceSetImpl();
	}

	/**
	 * Returns the factory associated with the model.
	 * Object creation are made through that factory.
	 * @return
	 */
	static public EMTFactory getFactory() {
		if (transformationModelFactory == null) {
			// Access the factory (needed to create instances)
			Map registry = EPackage.Registry.INSTANCE;
			String networkURI = EMTPackage.eNS_URI;
			EMTPackage transformationModelPackage =
				(EMTPackage) registry.get(networkURI);
			transformationModelFactory = transformationModelPackage.getEMTFactory();
		}
		return transformationModelFactory;
	}

	/**
	 * Creates a new transformation.
	 * @param 
	 * @return
	 */
	public Transformation createTransformation(IPath path) {
		createResource(path);
		// Create a new transformation model
		Map registry = EPackage.Registry.INSTANCE;
		String transformationmodelURI = EMTPackage.eNS_URI;
		EMTPackage nPackage = (EMTPackage) registry.get(transformationmodelURI);
		EMTFactory nFactory = nPackage.getEMTFactory();
		transformation = nFactory.createTransformation();
		resource.getContents().add(transformation);
		return transformation;
	}

	/**
	 * Loads the content of the model from the file.
	 * @param path
	 */
	public void load(IPath path) throws IOException {
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
		resource.load(options);
	}

	/**
	 * reloads the content of the model from the file.
	 * @param path
	 */
	/*public void reload(IPath path) throws IOException {
		getResource(path).unload();
		load(path);
	}*/

	/**
	 * Saves the content of the model to the file.
	 * @param path
	 */
	/*public void save(IPath path) throws IOException {
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
		resource.save(options);
	}*/

	/**
	 * Gets the top level element of transformation model.
	 * @return
	 */
	public Transformation getModel() {
		if (null == transformation) {
			EList l = resource.getContents();
			Iterator i = l.iterator();
			while (i.hasNext()) {
				Object o = i.next();
				if (o instanceof Transformation)
					transformation = (Transformation) o;
			}
		}
		return transformation;
	}
}

/**
 * $Log: TransformationModelManager.java,v $
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.4  2006/06/02 17:29:15  bunjip
 * Templates werden gefunden
 *
 * Revision 1.3  2006/05/31 13:25:46  bunjip
 * Anpassungen an neues Modell
 *
 * Revision 1.2  2006/05/24 13:04:36  bunjip
 * Code-Review durchgefuehrt - nicht benutzter Code auskommentiert
 *
 * Revision 1.1  2006/05/22 13:43:03  enrico
 * *** empty log message ***
 *
 * Revision 1.4  2006/04/25 12:40:34  bunjip
 * *** empty log message ***
 *
 * Revision 1.3  2006/04/25 12:37:25  bunjip
 * *** empty log message ***
 *
 * Revision 1.2  2006/03/20 01:26:59  bunjip
 * emfrule -> emftransformation
 *
 * Revision 1.1  2006/03/15 09:42:47  enrico
 * *** empty log message ***
 *
 * Revision 1.1  2006/03/14 17:06:10  eduardw
 * *** empty log message ***
 *
 * Revision 1.1  2005/12/20 15:33:59  enrico
 * initial version
 *
 */